(document:surround "/std/frame")
(document:insert "/std/functions")
(document:envelop with-translation _ "alterator-tzone")

;;; Logic

(define *countries* (make-cell '()))
(define *zones* (make-cell '()))

(define (default-index y lst)
  (or (list-index (lambda(x) (string=? (car x) y))
                  lst)))

(define (default-zone country)
  (let ((current (woo-get-option (woo-read-first (string-append "/timezone/" country))
                                  'zone)))
    (default-index current (cell-ref *zones*))))

(define (current-country)
  (car (list-ref (cell-ref *countries*)
                 (country-list current))))

(define (current-zone)
  (let ((current (zone-list current)))
    (and (>= current 0)
         (car (list-ref (cell-ref *zones*) current)))))

(define (update-zone-list)
  (woo-catch/message
   (thunk
    (let ((country (current-country)))
    (cell-set! *zones* (woo-list/name+label (string-append "/timezone/" country "/zones")))
    (zone-list rows (map cdr (cell-ref *zones*))
               current (default-zone country))))))

(define (write-zone . args)
  (woo-catch/message
   (thunk
    (let ((zone (current-zone)))
      (and zone (woo-write "/timezone"
                            'zone zone
                            'utc    (utc-set state)
                            'installer (not (null? args)) ))))))


;;; UI

(gridbox columns "30;40;30"
         ;; line
         (spacer)
         (vbox (label (_ "Please, select country:"))
               (document:id country-list (combobox)))
         (spacer)
         
         ;; line
         (spacer)
         (label " ")
         (spacer)
         
         ;; line
         (spacer)
         (vbox (label (_ "Please, select zone:"))
               (document:id zone-list (listbox)))
         (spacer)
	 
         ;; line
         (spacer)
         (document:id utc-set
                      (checkbox (_ "Hardware time set to GMT")))
         (spacer))

(or (global 'frame:next-button)
    (hbox (document:id apply-button (button (_ "Apply")))
          (button (_ "Quit") (when clicked (document:end)))))

;;; initialization

(document:root (when loaded
                 (woo-catch/message
                  (thunk
                   (country-list (when selected (update-zone-list)))
                   (apply-button (when clicked (write-zone)))

                   (let ((country-data (woo-list/name+label "/timezone"))
                         (current-data (woo-read-first "/timezone")))
                     (cell-set! *countries* country-data)
                     (utc-set state (woo-get-option current-data 'utc))
                     (country-list rows (map cdr country-data)
                                   current (default-index (woo-get-option current-data 'name) country-data)
                                   selected))))))

(frame:on-next (thunk (or (write-zone #t) 'cancel)))
(frame:on-back (thunk (or (write-zone #t) 'cancel)))

(and (global 'frame:auto-save)
     (cell-set! (global 'frame:auto-save) write-zone))
